//   Copyright 2012 Georg-August-Universität Göttingen, Germany
//
//   Licensed under the Apache License, Version 2.0 (the "License");
//   you may not use this file except in compliance with the License.
//   You may obtain a copy of the License at
//
//       http://www.apache.org/licenses/LICENSE-2.0
//
//   Unless required by applicable law or agreed to in writing, software
//   distributed under the License is distributed on an "AS IS" BASIS,
//   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
//   See the License for the specific language governing permissions and
//   limitations under the License.

package de.ugoe.cs.autoquest.commands.usability;

import java.util.List;

import de.ugoe.cs.autoquest.CommandHelpers;
import de.ugoe.cs.autoquest.tasktrees.treeifc.ITaskModel;
import de.ugoe.cs.autoquest.usability.UsabilityEvaluationManager;
import de.ugoe.cs.autoquest.usability.UsabilityEvaluationResult;
import de.ugoe.cs.util.console.Command;
import de.ugoe.cs.util.console.GlobalDataContainer;

/**
 * <p>
 * This command performs a usability evaluation based on a task tree. It uses the
 * {@link UsabilityEvaluationManager} for this purpose. Please consult the documentation of the
 * usability evaluation manager for more details.
 * </p>
 * 
 * @author Patrick Harms
 * @version 1.0
 */
public class CMDevaluateUsability implements Command {

    /*
     * (non-Javadoc)
     * 
     * @see de.ugoe.cs.util.console.Command#help()
     */
    @Override
    public String help() {
        return "evaluateUsability <tasktree> {<maxCount>} {<evaluationResult>}";
    }

    /*
     * (non-Javadoc)
     * 
     * @see de.ugoe.cs.util.console.Command#run(java.util.List)
     */
    @Override
    public void run(List<Object> parameters) {
        String tasktreeName = null;
        String evaluationResult = null;
        int maxCount = Integer.MAX_VALUE;
        try {
            for (Object parameter : parameters) {
                if (tasktreeName == null) {
                    tasktreeName = (String) parameter;
                }
                else {
                    try {
                        maxCount = Integer.parseInt((String) parameter);
                    }
                    catch (Exception e) {
                        if (evaluationResult == null) {
                            evaluationResult = (String) parameter;
                        }
                        else {
                            throw new IllegalArgumentException("unrecognized Parameter");
                        }
                    }
                }
            }
        }
        catch (Exception e) {
            throw new IllegalArgumentException("must provide a task tree name");
        }
        
        if (evaluationResult == null) {
            evaluationResult = "usabilityEvaluationResult";
        }

        Object dataObject = GlobalDataContainer.getInstance().getData(tasktreeName);
        if (dataObject == null) {
            CommandHelpers.objectNotFoundMessage(tasktreeName);
            return;
        }
        if (!(dataObject instanceof ITaskModel)) {
            CommandHelpers.objectNotType(tasktreeName, "ITaskTree");
            return;
        }

        ITaskModel taskTree = (ITaskModel) dataObject;
        
        UsabilityEvaluationResult result =
            new UsabilityEvaluationManager().evaluateUsability(taskTree, maxCount);
        
        if (GlobalDataContainer.getInstance().addData(evaluationResult, result)) {
            CommandHelpers.dataOverwritten(evaluationResult);
        }
        
    }

}
